%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [S]=GenScen(Prototype,NbSymb,ErrorRate, distErrorTypes, noOverlap)
% Random generation of a sequence based on a pattern prototype
%
%	Usage : 
%		Prototype : Pattern prototype (cf GenProto.m), required.
%		NbSymb : Available symbols number (available = that can be used to
%		generate symbols), required.
%		ErrorRate : Structural error rate, default 0.1
%		distErrorTypes : Distribution of structural error types (vector
%		3x1) (order : Insertion, mistake, omission), default [0.5;0.7;1].
%       noOverlap : if =0 possible overlaps between symbols, default 0
%
%	Returns a pattern S : matrix of size 3*? 
%       S(:,i) : the i-th symbol of S,
%       S(1,i) : the symbolic name of the i-th symbol
%       S(1,i) : the date of the i-th sy
%       S(1,i) : the duration of the i-th symbol
%
% Author : Guyet T. <thomas.guyet@irisa.fr>


Size=size(Prototype,2);
if nargin<3, ErrorRate=0.1; end
if nargin<4, distErrorTypes=[0.5;0.7;1]; end
if nargin<5, noOverlap=0; end

S=zeros(3,Size);

j=1;
for i=1:Size
   if (rand(1)<ErrorRate)
      %Error choice
      e=rand(1);
      if (e<distErrorTypes(1)) %Insertion of a random symbol
         S=cat(2,S,zeros(3,1));
         S(1,j+1)=Prototype(1,i);
         S(2,j+1)=abs(Prototype(2,i)+randn(1)*Prototype(3,i));
         S(3,j+1)=abs(Prototype(4,i)+randn(1)*Prototype(5,i));
         
         if (j~=1)
            if S(2,j+1)<(S(2,j-1)+S(3,j-1)) 
               S(2,j+1)=(S(2,j-1)+S(3,j-1))+rand(1)*Prototype(4,i)/5; % keep a temporal margin for the insertion
            end
            
         	S(1,j)=floor(rand(1)*NbSymb)+1;
	        S(2,j)=S(2,j-1)+S(3,j-1)+rand(1)*(S(2,j+1)-S(2,j-1)-S(3,j-1));
            S(3,j)=rand(1)*(S(2,j+1)-S(2,j));
         else
            S(1,j)=floor(rand(1)*NbSymb)+1;
	        S(2,j)=rand(1)*S(2,j+1);
            S(3,j)=rand(1)*(S(2,j+1)-S(2,j));
         end
         j=j+2;
      elseif (e<distErrorTypes(2)) % Mistake
         S(1,j)=floor(NbSymb*rand(1))+1;
         S(2,j)=abs(Prototype(2,i)+randn(1)*Prototype(3,i));
         
         if (j~=1 && noOverlap) %Non-overlapping constraint
             if ( S(2,j)<(S(2,j-1)+S(3,j-1)) )
                S(2,j)=S(2,j-1)+S(3,j-1);
             end
         end %Else, le absolute value ensure the begining coherence
         
         S(3,j)=abs(Prototype(4,i)+randn(1)*Prototype(5,i));
       	j=j+1;
      else 
	      % Symbol omission : delete one colonne from the matrix
			S=S(1:size(S,1),1:size(S,2)-1);          
      end
   else
      %Case no error !
   	 S(1,j)=Prototype(1,i);
       S(2,j)=abs(Prototype(2,i)+randn(1)*Prototype(3,i));
       
       if (j~=1 && noOverlap) %Non-overlapping constraint
          if S(2,j)<(S(2,j-1)+S(3,j-1))
             S(2,j)=S(2,j-1)+S(3,j-1);
          end
       end
       
       S(3,j)=abs(Prototype(4,i)+randn(1)*Prototype(5,i));
	   j=j+1;
   end
end
