function [kcentre,lambda,owner, sqrerr] = APCA(data, initialcentres, initialweights, distname)
% Adaptive Partitional Clustering Algorithm (APCA)
%
% Input
%   data = a matrix of size Lx2xN (N=number of intervals, L=dimensionality)
%   initialcentres = scalar value (i.e. the 'K' in K-means, how many clusters)
%                    or a 2xLxK matrix, defining the initial centres
%   initialweights = Lx1 vector defining the initial weights of the
%                    Weighted Hausdorff or city-block distance
%
% Implementation d'apres "Partitional clustering algorithms for symbolic
% interval data based on single adaptive distances", De Carvahlo,
% Lechevallier, Pattern Recognition 42:1223-1236, 2009
%
% Output
%   kcentre = 2xLxK matrix of final cluster centres
%   owner = Nx1 vector indicating which cluster a point belongs to
%   sqrerr = sum of squared errors (squared distances from nearest cluster centre)
%
% Author : Guyet T. <thomas.guyet@agrocampus-ouest.fr>
% GNU Licenced, Aout 2010

N = size(data,3); % num intervals
L = size(data,1); % num variables

disttype=3;
if( exist('distname', 'var') )
    if( strcmp(distname,'hausdorff') )
        %disp 'hausdorff'
        disttype=3;
    elseif( strcmp(distname,'cityblock2'))
        %disp 'cityblock2 : not implemented'
        disttype=2;
    elseif( strcmp(distname,'cityblock'))
        %disp 'cityblock'
        disttype=1;
    end
end

if (max(size(initialcentres))==1)
    % I've been given the scalar value K...setup centres at first K data
    % points
    initialcentres = data(:,:,1:min(N,initialcentres));
end

if (size(initialcentres,1) ~= L)
    %disp(L)
    %disp(initialcentres)
    %size(initialcentres)
    error('wrong size matrix: initialcentres ');
end

K = size(initialcentres,3); % num clusters

kcentre = initialcentres; % matrice Lx2xK contenant les prototypes pour chaque classe

if( size(initialweights,1) ~= L )
    error('wrong size matrix: initialweights');
end
lambda = initialweights;

owner = zeros(N,1); %vecteur attribuant un exemple (id) a une classe (valeur)

%% Initialisation : attribution al?atoire des exemples dans les K clusters
for i=1:N
    owner(i)=floor(random('unif', 1, K+1));
end

test=1;

while(test)
    %% Step 1 : Definition of the best prototypes (kcenter)
    for c=1:K
        total = sum(owner==c);
        if (total > 0) %il y a des exemples dans la classe
            kcentre(:, :, c) = median( data(:,:,owner==c), 3 );
        else
            kcentre(:, :, c) = median( kcentre(:, :, :), 3 );
            disp('disparition d une classe !');
        end
    end
    
    %% Step 2 : Definition of the best distance
    if( disttype == 3)  %(Hausdroff)
        num=1;
        for h=1:L
            part_num=0;
            for k=1:K
                i=find(owner==k);
                part_num = part_num + sum(max(abs(reshape( data(h,:,i), 2, size(i,1))-repmat(kcentre(h,:,k), size(i,1),1)')));
            end
            num = num * part_num;
        end
        num = num ^ (1/L);

        for h=1:L %for each dimension
            denum=0;
            for k=1:K
                i=find(owner==k);
                denum = denum + sum(max(abs(reshape( data(h,:,i), 2,size(i,1))-repmat(kcentre(h,:,k), size(i,1),1)')));
            end
            lambda(h)=num/denum;
        end
    elseif( disttype == 2)  %(Cityblock2)
        %not implemented
    elseif( disttype == 1)  %(Cityblock)
        num=1;
        for h=1:L
            part_num=0;
            for k=1:K
                i=find(owner==k);
                part_num = part_num + sum(sum(abs(reshape( data(h,:,i), 2,size(i,1))-repmat(kcentre(h,:,k), size(i,1),1)')));
            end
            num = num * part_num;
        end
        num = num ^ (1/L);

        for h=1:L %for each dimension
            denum=0;
            for k=1:K
                i=find(owner==k);
                denum = denum + sum(sum(abs(reshape( data(h,:,i), 2,size(i,1))-repmat(kcentre(h,:,k), size(i,1),1)')));
            end
            lambda(h)=num/denum;
        end
    end
    
    %% Step 3 : Definition of the best partition
    test = 0;
    sqrerr = 0;
    for i=1:N
        mini=inf;
        argmin=0;
        for k=1:K
            switch( disttype )
                case 3
                    d=Weighted_Hausdorff(data(:,:,i), kcentre(:,:,k), lambda);
                case 2
                    disp('not implemented');
                    %d=Weighted_Hausdorff(data(:,:,i), kcentre(:,:,k), lambda);
                case 1
                    d=Weighted_CityBlock(data(:,:,i), kcentre(:,:,k), lambda);
                otherwise
                    error('unknown distance type');
            end
               
            if( d < mini)
                mini=d;
                argmin=k;
            end
        end
        sqrerr = sqrerr + mini*mini;
        if( argmin ~= owner(i) )
            test=1;
            owner(i)=argmin;
        end
    end
    
    %% Stopping criterion : test
end

