function [kcentre,owner,sqrerr,bic] = kmeans_intervals(data, initialcentres, distance_function)
%
% [kcentre,owner,sqrerr] = kmeans(data, initialcentres, distance_function)
%
%
% Input:
%   data = a matrix of size Lx2xN (N=number of data points, L=dimensionality)
%   initialcentres = scalar value (i.e. the 'K' in K-means, how many clusters)
%                    or a KxL matrix, defining the initial centres
%   distance_function = the name of a distance function between two
%                       intervals
%
% Output:
%   kcentre = KxL matrix of final cluster centres
%   owner = Nx1 vector indicating which cluster a point belongs to
%   sqrerr = sum of squared errors (squared distances from nearest cluster centre)
%

N = size(data,3); % num intervals
L = size(data,1); % num variables
bic=inf;

if (max(size(initialcentres))==1)
    % I've been given the scalar value K...setup centres at first K data
    % points
    initialcentres = data(:,:,1:min(size(data,3),initialcentres));
end

if (size(initialcentres,1) ~= L)
    error('wrong size matrix: initialcentres');
end

K = size(initialcentres,3); % num clusters

kcentre = initialcentres; % K * L
ownerchange = 1;
owner = zeros(N,1);
while (ownerchange > 0)
    % Do clustering
    % compute squared error for each point
    sqrerr = 0;
    ownerchange = 0;
    for i = 1:N
        % find distance to each cluster centre
        best = 0;
        bestdist = inf;
        for j = 1:K
            distfn = str2func(distance_function);
            dist=distfn(data(:,:,i), kcentre(:,:,j));
            
            if (dist < bestdist)
                best=j;
                bestdist=dist;
            end
        end
        sqrerr = sqrerr + bestdist;

        % pick closest centre, and note if a point changes cluster
        if (owner(i) ~= best)
            owner(i) = best;
            ownerchange = 1;
        end
    end

    % Recalculate cluster centres for each cluster
    for i = 1:K
        total = sum(owner==i);
        % for each variable
        if (total > 0)
            kcentre(:,:,i) = median( data(:,:,owner==i), 3 );
        else
            % This cluster has no members!
            %Might generate a random cluster :
            
            %size(data(:,:,floor(rand*N)+1))
            %size((10^-10)*randn(L,2))
            kcentre(:,:,i) = data(:,:,floor(rand*N)+1)+(10^-10)*randn(L,2);
        end
    end

end

%% Compute bic criterion
L=1; %Likelihood (Gaussian distribution hypothesis)
for i = 1:K
    pos_i=data(:,1,owner==i);
    pos_i=reshape(pos_i, size(data, 1), size(pos_i,3))';
    Sigma=cov(pos_i);
    mu=mean(pos_i);
    for j = 1:size(pos_i,1)
        L = L * exp( -0.5*(pos_i(j,:)-mu)*Sigma*(pos_i(j,:)-mu)' )/( (2*pi)^(L/2)*det(Sigma)^(0.5) );
    end
end
bic = L - K*(L+1)/2*log(N);

