%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [P,patterns]=QTIAPriori(f, Ex, itMax, distance_threshold, StartLevel_TPruning)
% QTIAPriori main function
% [P,patterns]=QTIAPriori(f, Ex, itMax)
%	f : pattern frequence threshold (in [0,1])
%	E : Examples (cells that hold sequences). 
%		An sequence is an example : it must be a matrice (Ex) of size n*3, where n is the number of symbol in the sequence.
%		Ex(:,i) is the i-th symbol of the example Ex
%			Ex(1,i) is the symbolic name of the i-th symbol
%			Ex(2,i) is the date of the i-th symbol
%			Ex(3,i) is the duration of the i-th symbol
%	
%   itMax : Maximal number of iterations
%  returns 
%   P : the extracted patterns
%	patterns : cells of extracted patterns at all the APriori algorithm steps.
%
% The main function of the QTIAPriori algorithm is based on the APrioriTID
% framework (Aggrawal and Srikant, 96).
% It's a growing pattern algorithm : at the n-th step, frequent temporal 
% patterns of size n are created based on the frequent temporal patterns 
% of size n-1.
%
% Author : Guyet T. <thomas.guyet@irisa.fr>

global TPC_algo;
global distance_function;
global nbClassif;

%% Creation of the list of frequent symbols and associated patterns of size
%% 1

patterns=cell(itMax,1);

%Construct the initial location based on the 1D patterns
P=[];
for i=1:size(Ex,1)
    e=Ex{i};
    
    for j=1:size(e,2)
        s.ExId = i;
        s.pos = j;
        s.is = e(:,j);
        
        %Update the list (without copy itemset)
        ok=0;
        for k=1:size(P)
            itemset = P{k,1};
            if( itemset(1,1) == e(1,j) )
                %this item already exists
                ok=1;
                itemloc=[P{k,2} s];
                P(k,2) = {itemloc};
                break;
            end
        end
        if(~ok)
            initloc{1}=s;
            P=[P; {[e(1,j);0;0]},{initloc}];
        end
    end
end

%% Construction of the symbol list
listSymbols = cell2mat(P(:,1));
listSymbols=listSymbols(1:3:size(listSymbols));

%% Main loop
k=1;
while(~isempty(P) && k<itMax)
    newP=[];
    
    if( k >= StartLevel_TPruning )
        %%Temporal constraints construction
        for i=1:size(P,1)
            itemset = P{i,1};
            itemloc = P{i,2};

            %% HACK : CLASSIFICATION MUST BE IMPROVED USING TEMPORAL POSITION
            %% AS A STARTING POINT FOR CLASSIFICATION ??

            switch(TPC_algo)
                case 'EM'
                    Mr = EM_TemporalPatternConstruction(itemset, itemloc);
                case 'KMeans'
                    [Mr, owner] = KMeans_TemporalPatternConstruction(itemset, itemloc);
                case 'AP'
                    [Mr, owner] = AP_TemporalPatternConstruction(itemset, itemloc);
                case 'DynaClust'
                    [Mr, owner] = DynaClust_TemporalPatternConstruction(itemset, itemloc);
                otherwise
                    Mr=[];
            end
            nbClassif=nbClassif+1;

            if( isempty(Mr) ) 
                continue;
            end

            for idMr=1:size(Mr,3) % for all temporal patterns identified
                new_pattern = Mr(:,:,idMr);

                % Solution 1 : Distribution of the itemloc that matches the temporal
                % patterns
                itemloc_Mr={};
                for ide = 1:size(itemloc,2)
                    if( comparePatterns(itemloc{ide}.is, new_pattern, distance_function, distance_threshold) )
                        itemloc_Mr=[itemloc_Mr itemloc{ide}];
                    end
                end

                %Solution 2 : Use the classification result
%                 itemloc_Mr=itemloc(owner==idMr);


                % Check if the pattern has not already be included (it happens
                % ?)
                ok=0;
                for idp = 1:size(newP,1)
                    if( comparePatterns(newP{idp,1}, new_pattern, distance_function, distance_threshold) )
                        ok=1;
                        break;
                    end
                end
                if( ok )
                    % Here: itemlocs of new_pattern and newP{idp,1} are merged
                    % (without copy)
                    % merged itemlocs are not ordered !!
                    extendable_itemloc=newP{idp,2};
                    toAdd=[];
                    for id=1:size(itemloc_Mr,2)
                        found=0;
                        cur_itemloc=itemloc_Mr{id};
                        for pos=1:size(extendable_itemloc,2)
                            p=extendable_itemloc{pos};
                            if( p.ExId == cur_itemloc.ExId && p.pos == cur_itemloc.pos )
                                if( comparePatterns(p.is, cur_itemloc.is, distance_function, distance_threshold) )
                                    found=1;
                                    break;
                                end
                            end
                        end
                        if( ~found )
                            toAdd=[toAdd itemloc_Mr(id)];
                        %else % It happens even with classification result
                        %    disp('======================= FOUND ======================');
                        end
                    end

                    newP{idp,2}=[newP{idp,2} toAdd];
                    continue
                end

                if( size(itemloc_Mr,2)<f*size(Ex,1) )
                    continue;
                end

                newP=[newP; {new_pattern} {itemloc_Mr}];
            end
        end
    else
        newP=P;
    end
    
    if( isempty(newP) )
        break;
    end
    patterns(k)={newP};
    
    % HERE newP holds frequent temporal patterns of size k
    
    % Generation of new candidates
    P=generateCandidates(newP,listSymbols, distance_function, distance_threshold);
    newP=[];
    % Mise a jour des structures de donnees et elagage structurel !
    for i=1:size(P,1)
        s=P{i,2};
        itemloc={};
        for j=1:size(s,2)
            sst = Ex{s{j}.ExId};
            pos = s{j}.pos+1;
            while( pos <= size(sst,2) )
                if( sst(1,pos) == P{i,1}(1,size(P{i,1},2)) )
                    ns.ExId=s{j}.ExId;
                    ns.is= cat(2,s{j}.is, sst(:,pos));
                    ns.pos = pos;
                    itemloc=[itemloc ns];
                end
                pos=pos+1;
            end
        end
        P(i,2) = {itemloc};
        
        %Elagage !
        if( size(itemloc,2)>=f*size(Ex,1) )
            newP=[newP; P(i,:)];
        end
    end
    P=newP;
    
    if( isempty(P) )
        break;
    end
    k=k+1;
end

%% Final case
if( ~isempty(P) )
    newP=[];
    %Temporal constraints construction
    for i=1:size(P,1)
        itemset = P{i,1};
        itemloc = P{i,2};
        
        switch(TPC_algo)
            case 'EM'
                Mr = EM_TemporalPatternConstruction(itemset, itemloc);
            case 'KMeans'
                Mr = KMeans_TemporalPatternConstruction(itemset, itemloc);
            case 'AP'
                Mr = AP_TemporalPatternConstruction(itemset, itemloc);
            case 'DynaClust'
                Mr = DynaClust_TemporalPatternConstruction(itemset, itemloc);
            otherwise
                Mr=[];
        end
        nbClassif=nbClassif+1;
        
        if( isempty(Mr) ) 
            continue;
        end
        
        for idMr=1:size(Mr,3) % for all temporal patterns identified
            new_pattern = Mr(:,:,idMr);
            
            %% HACK : MUST BE IMPROVED USING idx INFORMATION FROM
            %% CLASSIFICATION ALGORITHMS
            
            % Distribution of the itemloc that matches the temporal
            % patterns
            itemloc_Mr={};
            for ide = 1:size(itemloc,2)
                if( comparePatterns(itemloc{ide}.is, new_pattern, distance_function, distance_threshold) )
                    itemloc_Mr=[itemloc_Mr itemloc{ide}];
                end
            end

            if( size(itemloc_Mr,2)<f*size(Ex,1) )
                continue;
            end
            
            newP=[newP; {new_pattern} {itemloc_Mr}];
        end
    end
    P=newP;
    
    if(isempty(newP))
        if(k>1), k=k-1; end
    else
        patterns(k)={P};
    end
end

% Calcul de la bordure
Bordure={};
if( k>1 )
    for l=k:-1:1 %Par taille de motif d?croissante
        for idp=1:size(patterns{l},1)
            pat=patterns{l};
            pat=pat{1};
            ok=0;
            for idb=1:size(Bordure,2)
                if( similarSubPattern(pat, Bordure{idb}, distance_function, distance_threshold) )
                    ok=1;
                    break;
                end
            end
            if( ok )
                continue;
            end
            %Ajout a la bordure si il n'a pas de super motif
            Bordure=[Bordure pat];
        end
    end
end
P=Bordure;


