%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function P=QTIPrefixSpanRec(f, Ex, tMax, distance_threshold, StartLevel_TPruning, L1, prefix, prefixloc)
%	Recursive function implementing the PrefixSpan algorithm with
%	quantitative temporal interval extraction
%   Note that this function must not be called directly! Use the
%   QTIPrefixSpan function (recursion initialisation function)
%   P=QTIPrefixSpanRec(f, Ex, d, tMax, L1, prefix, prefixloc)
%   f : pattern frequence threshold (in [0,1])
%	Ex : Examples (cells that hold sequences). 
%		An sequence is an example : it must be a matrice (Ex) of size n*3, 
%       where n is the number of symbol in the sequence.
%		Ex(i,:) is the i-th symbol of the example Ex
%			Ex(i,1) is the symbolic name of the i-th symbol
%			Ex(i,2) is the date of the i-th symbol
%			Ex(i,3) is the duration of the i-th symbol
%			
%	d : density (histogram creation parameters)
%   epsilon : item similarity threshold
%   tMax : Maximal length of itemsets
%   L1 : frequent items (can be postfixed)
%   prefix : the temporal pattern to extend
%   prefixloc : structured localisation of the prefixes : a cell containing
%   couples (ExId,pos,is)
%       is : an example of itemset (subset of an symbolic interval serie) 
%       ExId : symbolic serie index
%       pos : position of the last item of the itemset in the symbolic
%           serie
%   
%   returns 
%       P : the extracted patterns
%
% Author : Guyet T. <thomas.guyet@irisa.fr>


global G_explored_patterns; % The set of explored patterns
global prune_super_explored;
global TPC_algo;
global distance_function;
global nbClassif; % Total number of classification that have been required !
P={};

if( size(prefix,2)>=tMax )  % Stop the recursion if the prefix length is 
                            % upper than tMax
    P={prefix};
    return;
end


%Try to extend the prefix with one of the frequent items
for i=1:size(L1,1)
    item = L1(i);   % item is the postfixed item
    nbis=0;         % nbis is the number of occurrences of {prefix+item}
    itemloc={};     % cells of locations for the postfixed item
    for loc=1:size(prefixloc,2)
        s=prefixloc{loc};
        pos = s.pos+1;
        e=Ex{s.ExId}; %e is a sequence in which prefix occurs before 
                      %the position pos
        
        % Looking for the item after the position of the prefix (pos) in
        % the sequence e
        found=0;
        while( pos<=size(e,2) )
            if( e(1,pos)==item ) % symbolicaly matching item
                found=1;
                s.is = cat(2, s.is, e(:,pos)); %extends the matching pattern
                break;
            end
            pos=pos+1;
        end
        
        if(found)
            nbis=nbis+1;
            s.pos=pos;
            itemloc=[itemloc s];
        end
    end
    
    if( nbis<f*size(Ex,1) )
        continue;
    end
    
    % HERE : {prefix + item} is a frequent sequence (no temporal
    % constraints)

    itemset =cat(2,prefix,[item;0;0]);
    
    if( size(itemset,2) >= StartLevel_TPruning )
        %% Temporal constraints construction
        switch(TPC_algo)
            case 'EM'
                Mr = EM_TemporalPatternConstruction(itemset, itemloc);
            case 'KMeans'
                [Mr, owner] = KMeans_TemporalPatternConstruction(itemset, itemloc);
            case 'AP'
                [Mr, owner] = AP_TemporalPatternConstruction(itemset, itemloc);
            case 'DynaClust'
                [Mr, owner] = DynaClust_TemporalPatternConstruction(itemset, itemloc);
            case 'Separate'
                [Mr, owner] = Separation_TemporalConstruction(itemset, itemloc);
            otherwise
                Mr=[];
        end
        nbClassif=nbClassif+1;

        if( isempty(Mr) )
            continue;
        end
        
        %Concatenate the results and recursion
        for idMr=1:size(Mr,3) % for all temporal patterns identified
            new_pattern = Mr(:,:,idMr);

            if(prune_super_explored)
                %Check if super patterns have already been explored
                already_explored=0;
                for id_gep = 1:size(G_explored_patterns,2)
                    if( similarSubPattern(new_pattern, G_explored_patterns{id_gep}, distance_function, distance_threshold) )
%                        disp('===================');
%                        disp(new_pattern);
%                        disp('      by');
%                        disp(G_explored_patterns{id_gep});
%                        disp('===================');
                       already_explored=1;
                       break;
                   end
                end
                if(already_explored)
                    continue;
                end
            end
            
            %If super patterns have not already, then new_pattern is in the
            %bordure : he is added to G_explored_patterns
            
            if( strcmp(TPC_algo,'Separate') )
            	% we do not use the distance with patterns to assign an example to a pattern
                itemloc_Mr=itemloc(owner==idMr);
            else
		        % Solution 1 :Distribution of the itemloc that matches the temporal
		        % patterns (soft contraint)
		        itemloc_Mr={};
		        for ide = 1:size(itemloc,2)
		            if( comparePatterns(itemloc{ide}.is, new_pattern, distance_function, distance_threshold) )
		                itemloc_Mr=[itemloc_Mr itemloc{ide}];
		            end
		        end

		        % Solution 2 : Use the classification result (hard constraint,
		        % more efficient)
				% itemloc_Mr=itemloc(owner==idMr);
			end

            if( size(itemloc_Mr,2)<f*size(Ex,1) )
                continue;
            end
            %Add the new_pattern to the explored ones
            % 27/10/2010 : TODO deplacement avant verification de frequence ???
            % : ne semble pas plus efficace ???
            G_explored_patterns=[G_explored_patterns new_pattern];
        
            %Recursion:
            ExtendedPatterns=QTIPrefixSpanRec(f, Ex, tMax, distance_threshold, StartLevel_TPruning, L1, new_pattern, itemloc_Mr);
            P=[P ExtendedPatterns];
        end
    else % Do not construct temporal constraints!
        % Distribution of the itemloc that matches the pattern
        itemloc_Mr={};
        for ide = 1:size(itemloc,2)
            if( comparePatterns(itemloc{ide}.is, itemset, distance_function, distance_threshold) )
                itemloc_Mr=[itemloc_Mr itemloc{ide}];
            end
        end
        
        % Recursion
        ExtendedPatterns=QTIPrefixSpanRec(f, Ex, tMax, distance_threshold, StartLevel_TPruning, L1, itemset, itemloc_Mr);
        P=[P ExtendedPatterns];
    end
    
end

if( isempty(P) )
    %Not any frequent pattern has been founded out, then prefix is
    %in the bordure
    P={prefix};
end
