%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [argmin, mindist] = extractPattern(p, s, distance_function, alignment_type, alignment_decay)
% Function to know if the pattern p is within a sequence s according to a given interval distance
% function.
% [b, val] = extractPattern(p, s, distance_function, threshold)
%
%   If several alignment are possible, the function try all of them to give
%   the alignment with the lowest distance with the threshold.
%	The temporal 
%   Be careful, it's a quite tricky algorithm.
%
% input
%   p: a patterns (matrix of size 3 x length of the pattern)
%	s: a sequence in which find the pattern p (matrix of size 3 x length of s)
%   distance_function: the name of a distance function between intervals
%   (eg 'CityBlock', 'Hausdorff')
%   threshold (default: inf): an interval similarity threshold (inf if not given)
%	alignment_type
%		- 0 (default), the timestamps are aligned according to the date of the first event
%		- 1, the timestamps are aligned according to the date of the last event
%	alignment_decay (default 0)
%
%
% output:
%	argmin: the positions defining the closest sub-sequence in s of the pattern p
%   mindist : the value of the distance between closest sub-sequence in s and the pattern p
%
% Author : Guyet T. <thomas.guyet@irisa.fr>


if( nargin < 4)
    alignment_type=0;
end
if( nargin < 5)
    alignment_decay=0;
end

argmin=[];
mindist=inf;

if( size(p,2)>size(s,2) ) %%pattern is longer than the sequence: no possible match
    return;
end

c=ones(size(p,2),1); %c is an index vector : c(i) gives the index of an item of s that is equals to the i-th item of p

%distance function from the 
distfn=str2func(distance_function);

test=1;
while(test)
    config_found=1;
    for i=1:size(c,1)
        j=c(i);
        while( j<=size(s,2) && p(1,i)~=s(1,j) )
            j=j+1;
        end
        if( j>size(s,2) || p(1,i)~=s(1,j) ) %HACK possibilite de s'arreter lorsque je ne laisse plus suffisamment de place au reste du motif !
            config_found=0;
            break;
        end
        
        %On donne la bonne valeur a c(i)
        c(i)=j;
        
        % On repousse les suivants en utilisant l'ordre des items
        % If values have already been setted, we don't change it
        for k=(i+1):size(c,1)
            if( c(k)<c(k-1)+1 )
                c(k)=c(k-1)+1;
            end
        end
    end
    if( ~config_found )
        break;
    end
    % HERE : we found a good mapping (ie same symbolic signature) 
    % between the pattern and a sub-sequence:
    %% We compare the corresponding intervals
    
    % construction of the multi-dimensionnal intervals
    I1(:,1)=p(2,:)';
    I1(:,2)=p(2,:)'+p(3,:)';
    I2(:,1)=s(2,c)'; %extract only the intervals corresponding to the symbolic signature of p
    I2(:,2)=s(2,c)'+s(3,c)';
    
    % temporal alignment of the sequence interval on the pattern interval
    if( alignment_type == 0 ) %beginning alignment
    	decay=I1(1,1) - I2(1,1) + alignment_decay;
    else
    	l=size(I1,1);
    	decay=I1(l,1) - I2(l,1) + alignment_decay;
    end
    I2 = I2+decay;
    
    % distance computing
    dist = distfn(I1, I2);
    if( dist < mindist)
        mindist = dist;
        argmin=c;
    end
    
    % Now, if possible, we modify the matching alignment
    % to force the algorithm to find another one.
    % Just one position is modified, the most possible on the right
    % If not possible, test=0 and it stops the while
    test=0;
    for i=size(c,1):-1:1
        if( c(i) < (size(s,2)-i+1) )
            c(i)=c(i)+1;
            test=1;
            break
        end
    end
    
end


